# Copyright 2025 Flower Labs GmbH. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Abstract base class for ArtifactProvider."""


from abc import ABC, abstractmethod


class ArtifactProvider(ABC):
    """ArtifactProvider interface for providing artifact download links."""

    @abstractmethod
    def get_url(self, run_id: int) -> str | None:
        """Return the artifact download link for the given run ID."""

    @property
    @abstractmethod
    def output_dir(self) -> str:
        """Permanent storage directory."""

    @property
    @abstractmethod
    def tmp_dir(self) -> str:
        """Temporary storage directory."""
