"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright 2022 Flower Labs GmbH. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
==============================================================================
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Node(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NODE_ID_FIELD_NUMBER: builtins.int
    node_id: builtins.int
    def __init__(
        self,
        *,
        node_id: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["node_id", b"node_id"]) -> None: ...

global___Node = Node

@typing.final
class NodeInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NODE_ID_FIELD_NUMBER: builtins.int
    OWNER_AID_FIELD_NUMBER: builtins.int
    OWNER_NAME_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    REGISTERED_AT_FIELD_NUMBER: builtins.int
    LAST_ACTIVATED_AT_FIELD_NUMBER: builtins.int
    LAST_DEACTIVATED_AT_FIELD_NUMBER: builtins.int
    UNREGISTERED_AT_FIELD_NUMBER: builtins.int
    ONLINE_UNTIL_FIELD_NUMBER: builtins.int
    HEARTBEAT_INTERVAL_FIELD_NUMBER: builtins.int
    PUBLIC_KEY_FIELD_NUMBER: builtins.int
    node_id: builtins.int
    owner_aid: builtins.str
    owner_name: builtins.str
    status: builtins.str
    registered_at: builtins.str
    last_activated_at: builtins.str
    last_deactivated_at: builtins.str
    unregistered_at: builtins.str
    online_until: builtins.float
    heartbeat_interval: builtins.float
    public_key: builtins.bytes
    def __init__(
        self,
        *,
        node_id: builtins.int = ...,
        owner_aid: builtins.str = ...,
        owner_name: builtins.str = ...,
        status: builtins.str = ...,
        registered_at: builtins.str = ...,
        last_activated_at: builtins.str | None = ...,
        last_deactivated_at: builtins.str | None = ...,
        unregistered_at: builtins.str | None = ...,
        online_until: builtins.float | None = ...,
        heartbeat_interval: builtins.float = ...,
        public_key: builtins.bytes = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["_last_activated_at", b"_last_activated_at", "_last_deactivated_at", b"_last_deactivated_at", "_online_until", b"_online_until", "_unregistered_at", b"_unregistered_at", "last_activated_at", b"last_activated_at", "last_deactivated_at", b"last_deactivated_at", "online_until", b"online_until", "unregistered_at", b"unregistered_at"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["_last_activated_at", b"_last_activated_at", "_last_deactivated_at", b"_last_deactivated_at", "_online_until", b"_online_until", "_unregistered_at", b"_unregistered_at", "heartbeat_interval", b"heartbeat_interval", "last_activated_at", b"last_activated_at", "last_deactivated_at", b"last_deactivated_at", "node_id", b"node_id", "online_until", b"online_until", "owner_aid", b"owner_aid", "owner_name", b"owner_name", "public_key", b"public_key", "registered_at", b"registered_at", "status", b"status", "unregistered_at", b"unregistered_at"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_last_activated_at", b"_last_activated_at"]) -> typing.Literal["last_activated_at"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_last_deactivated_at", b"_last_deactivated_at"]) -> typing.Literal["last_deactivated_at"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_online_until", b"_online_until"]) -> typing.Literal["online_until"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_unregistered_at", b"_unregistered_at"]) -> typing.Literal["unregistered_at"] | None: ...

global___NodeInfo = NodeInfo
